local dot_dist = tonumber(minetest.settings:get("dot_dist")) or 60
local update_interval = tonumber(minetest.settings:get("radar_update_interval")) or 1.5
local play_sounds = tonumber(minetest.settings:get("play_sounds")) or false
local y_pos = tonumber(minetest.settings:get("y_pos")) or 0.45
local x_offset = tonumber(minetest.settings:get("x_offset")) or 200
local y_offset = tonumber(minetest.settings:get("y_offset")) or 0

manhunt = {}
manhunt.hunters = {}
manhunt.huntee = ""
manhunt.manhunt = false
manhunt.huds = {}
manhunt.points = {}
manhunt.alts = {}

local function hasValue(tbl, value)
   for k, v in ipairs(tbl) do -- iterate table (for sequential tables only)
      if v == value or (type(v) == "table" and hasValue(v, value)) then
	 return true -- Found in this or nested table
      end
   end
   return false -- Not found
end

function indexOf(array, value)
    for i, v in ipairs(array) do
        if v == value then
            return i
        end
    end
    return nil
end

local function activate_closure(stack, player)
   player_name = player:get_player_name()
   if stack:get_name() == "manhunt:compass_off" then
      manhunt.huds[player_name] = player:hud_add({
	    hud_elem_type = "image",
	    text = "manhunt_radar_background.png",
	    position = {x = 0, y = y_pos},
	    scale = {x = 3, y = 3},
	    alignment = {x = 0, y = 0},
	    offset = {x = x_offset, y = y_offset},
      })

      if manhunt.manhunt == true then
	 manhunt.points[player_name] = player:hud_add({
	       hud_elem_type = "image",
	       text = "manhunt_radar_point.png",
	       position = {x = 0, y = y_pos},
	       scale = {x = 3, y = 3},
	       alignment = {x = 0, y = 0},
	       offset = {x = x_offset, y = y_offset},
	 })
	 manhunt.alts[player_name] = player:hud_add({
	       hud_elem_type = "image",
	       text = "manhunt_radar_eq.png",
	       position = {x = 0, y = y_pos},
	       scale = {x = 3, y = 3},
	       alignment = {x = 0, y = 0},
	       offset = {x = x_offset, y = y_offset},
	 })
      end
      minetest.sound_play("activate_closure", {object = player})
      stack:set_name("manhunt:compass_on")
   elseif stack:get_name() == "manhunt:compass_on" then

      minetest.sound_play("deactivate_closure", {object = player})
      stack:set_name("manhunt:compass_off")
      if manhunt.huds[player_name] == nil then
	 return stack
      end
      player:hud_remove(manhunt.huds[player_name])
      if manhunt.points[player_name] ~= nil then
	 player:hud_remove(manhunt.points[player_name])
      end
      if manhunt.alts[player_name] ~= nil then
	 player:hud_remove(manhunt.alts[player_name])
      end
   end
   return stack
end

local function calc_compass_direction()
   if manhunt.manhunt == false then
      minetest.after(update_interval, calc_compass_direction)
      return
   end

   local huntee = minetest.get_player_by_name(manhunt.huntee)
   if not huntee then
      minetest.after(update_interval, calc_compass_direction)
      return
   end
   
   local huntee_pos = huntee:get_pos()
   for _, huntername in pairs(manhunt.hunters) do
      
      if huntername == "" then
	 minetest.after(update_interval, calc_compass_direction)
	 return
      end
      
      local hunter = minetest.get_player_by_name(huntername)

      if not hunter then
	 minetest.after(update_interval, calc_compass_direction)
	 return
      end
      if manhunt.huds[huntername] == nil then
	 minetest.after(update_interval, calc_compass_direction)
	 return
      end
      
      local inv = hunter:get_inventory()
      if manhunt.huds[huntername] == nil then
	 goto continue2
      end
      
      for _, i in pairs(inv:get_list("main")) do
	 if string.match(i:get_name(), "manhunt:compass_on") then
	    local hyaw = hunter:get_look_horizontal()
	    local hpos = hunter:get_pos()
	    local dist = math.sqrt((huntee_pos.x-hpos.x)^2 + (huntee_pos.z-hpos.z)^2)
	    local dotpos = {x = 0, y = 0}
	    local alt = ""
	    if dist < 3 then
	       if math.floor(hpos.y) < math.floor(huntee_pos.y) then alt = "manhunt_radar_up.png"
	       elseif math.floor(hpos.y) > math.floor(huntee_pos.y) then alt = "manhunt_radar_down.png"
	       elseif math.floor(hpos.y) == math.floor(huntee_pos.y) then alt = "manhunt_radar_eq.png"
	       end
	       dotpos = {x = x_offset, y = y_offset}
	       if manhunt.points[hunter:get_player_name()] == nil then
		  manhunt.points[hunter:get_player_name()] = hunter:hud_add({
	       hud_elem_type = "image",
	       text = "manhunt_radar_point.png",
	       position = {x = 0, y = y_pos},
	       scale = {x = 3, y = 3},
	       alignment = {x = 0, y = 0},
	       offset = {x = x_offset, y = y_offset},
		  })
	       end
	       if manhunt.alts[hunter:get_player_name()] == nil then
		  manhunt.alts[hunter:get_player_name()] = hunter:hud_add({
	       hud_elem_type = "image",
	       text = "manhunt_radar_eq.png",
	       position = {x = 0, y = y_pos},
	       scale = {x = 3, y = 3},
	       alignment = {x = 0, y = 0},
	       offset = {x = x_offset, y = y_offset},
		  })
	       end
	       hunter:hud_change(manhunt.points[hunter:get_player_name()], "offset", dotpos)
	       hunter:hud_change(manhunt.alts[hunter:get_player_name()], "text", alt)
	       if play_sounds == true then
		       minetest.sound_play("radar_alarm", {object = hunter})
	       end
	       
	       goto continue2
	    else
	       dotpos.x = (huntee_pos.x - hpos.x)/dist
	       dotpos.y = (huntee_pos.z - hpos.z)/dist
	       local temporaryx = dotpos.x
	       dotpos.x = ((dotpos.x) * math.cos(hyaw) + (dotpos.y) * math.sin(hyaw))
	       dotpos.y = (-((temporaryx) * math.sin(hyaw)) + ((dotpos.y) * math.cos(hyaw)))
	       temporaryx = nil
	       dotpos.x = (dotpos.x * 60) + x_offset
	       dotpos.y = (-dotpos.y * 60)
	       if math.floor(hpos.y) < math.floor(huntee_pos.y) then alt = "manhunt_radar_up.png"
	       elseif math.floor(hpos.y) > math.floor(huntee_pos.y) then alt = "manhunt_radar_down.png"
	       elseif math.floor(hpos.y) == math.floor(huntee_pos.y) then alt = "manhunt_radar_eq.png"
	       end
	       
		  if manhunt.points[hunter:get_player_name()] == nil then
		     manhunt.points[hunter:get_player_name()] = hunter:hud_add({
			   hud_elem_type = "image",
			   text = "manhunt_radar_point.png",
			   position = {x = 0, y = y_pos},
			   scale = {x = 3, y = 3},
			   alignment = {x = 0, y = 0},
			   offset = {x = x_offset, y = y_offset},
		     })
		  end
		  if manhunt.alts[hunter:get_player_name()] == nil then
		     manhunt.alts[hunter:get_player_name()] = player:hud_add({
			   hud_elem_type = "image",
			   text = "manhunt_radar_eq.png",
			   position = {x = 0, y = y_pos},
			   scale = {x = 3, y = 3},
			   alignment = {x = 0, y = 0},
			   offset = {x = x_offset, y = y_offset},
		     })
		  end
	       

		  hunter:hud_change(manhunt.points[hunter:get_player_name()], "offset", dotpos)
		  hunter:hud_change(manhunt.alts[hunter:get_player_name()], "text", alt)
	       if play_sounds == true then
		  minetest.sound_play("radar_update", {object = hunter})
	       end
	       
	    end
	    hunter:hud_change(manhunt.points[hunter:get_player_name()], "offset", dotpos)
	    hunter:hud_change(manhunt.alts[hunter:get_player_name()], "text", alt)
	    goto continue2
	 else
	    goto continue
	    minetest.after(update_interval, calc_compass_direction)
	 end
	 ::continue::
      end
      ::continue2::
   end
   minetest.after(update_interval, calc_compass_direction)   
end

minetest.register_on_respawnplayer(function(player)
      if not player then
	 return
      end
      
      if hasValue(manhunt.hunters, player:get_player_name()) then
	 if manhunt.points[player:get_player_name()] ~= nil then
	    if not player then
	       return
	    end
	    player:hud_remove(manhunt.points[player:get_player_name()])
	    manhunt.points[player:get_player_name()] = nil
	 end
	 if manhunt.huds[player:get_player_name()] ~= nil then
	    if not player then
	       return
	    end
	    player:hud_remove(manhunt.huds[player:get_player_name()])
	    manhunt.huds[player:get_player_name()] = nil
	 end
	 if manhunt.alts[player:get_player_name()] ~= nil then
	    if not player then
	       return
	    end
	    player:hud_remove(manhunt.alts[player:get_player_name()])
	    manhunt.alts[player:get_player_name()] = nil
	 end
	 if player:get_inventory():contains_item("main", "manhunt:compass_off") == false and player:get_inventory():contains_item("main", "manhunt:compass_on") == false then
	    player:get_inventory():add_item("main", ItemStack("manhunt:compass_off"))
	 end
      end
end)

minetest.register_craftitem("manhunt:compass_on", {
			       description = "The manhunt compass",
			       inventory_image = "manhunt_compass_on.png",
			       on_use = activate_closure,
			       on_drop = function(itemstack, dropper, pos)
				  newstack = activate_closure(itemstack, dropper)
				  minetest.item_drop(newstack, dropper, pos)
				  return newstack
			       end,
			       groups = {
				  not_in_creative_inventory = 1,
			       },
			       stack_max = 1,

			       
})

minetest.register_craftitem("manhunt:compass_off", {
			       description = "The manhunt compass",
			       inventory_image = "manhunt_compass_off.png",
			       on_use = activate_closure,
			       stack_max = 1,
})

minetest.register_craft({
	output = "manhunt:compass_off",
	recipe = {
		{"default:tin_ingot", "default:copper_ingot", "default:tin_ingot"},
		{"default:stick", "default:glass", "default:stick"},
		{"default:stick", "default:steel_ingot", "default:stick"},
	}
})

minetest.register_on_leaveplayer(function(player, timed_out)
      if not player then
	 return
      end
      
      if hasValue(manhunt.hunters, player:get_player_name()) then
	 if manhunt.points[player:get_player_name()] ~= nil then
	    if not player then
	       return
	    end
	    player:hud_remove(manhunt.points[player:get_player_name()])
	    manhunt.points[player:get_player_name()] = nil
	 end
	 if manhunt.huds[player:get_player_name()] ~= nil then
	    if not player then
	       return
	    end
	    player:hud_remove(manhunt.huds[player:get_player_name()])
	    manhunt.huds[player:get_player_name()] = nil
	 end
	 if manhunt.alts[player:get_player_name()] ~= nil then
	    if not player then
	       return
	    end
	    player:hud_remove(manhunt.alts[player:get_player_name()])
	    manhunt.alts[player:get_player_name()] = nil
	 end
	 if player:get_inventory():contains_item("main", "manhunt:compass_off") == false and player:get_inventory():contains_item("main", "manhunt:compass_on") == false then
	    player:get_inventory():add_item("main", ItemStack("manhunt:compass_off"))
	 end
      end
end)

minetest.register_on_joinplayer(function(player, last_login)
      if not player then return end
      if manhunt.huds[player:get_player_name()] ~= nil then return end
      if player:get_inventory():contains_item("main", "manhunt:compass_on") then
	 manhunt.huds[player:get_player_name()] = player:hud_add({
	       hud_elem_type = "image",
	       text = "manhunt_radar_background.png",
	       position = {x = 0, y = y_pos},
	       scale = {x = 3, y = 3},
	       alignment = {x = 0, y = 0},
	       offset = {x = x_offset, y = y_offset}
	 })
      end
end)

minetest.register_privilege("manhuntadmin", {
			       description = "Can moderate the manhunt and control the game.",
})

minetest.register_chatcommand("sethuntee", {
				 privs = {
				    manhuntadmin = true,
				 },
				 func = function(name, param)
				    if hasValue(minetest.get_connected_players(), minetest.get_player_by_name(param)) then
				       minetest.chat_send_player(name, param .. " has been successfully set as the huntee.")
				       manhunt.huntee = param
				    elseif hasValue(manhunt.hunters, param) then
				       minetest.chat_send_player(name, "A hunter cannot be a huntee at the same time.")
				       
				       return
				    else
				       minetest.chat_send_player(name, param .. " is either not online or already a huntee.")
				       manhunt.huntee = ""
				       return
				    end
				    
				    
end})

minetest.register_chatcommand("removehuntee", {
				 privs = {
				    manhuntadmin = true,
				 },
				 func = function(name, param)
				    if param == manhunt.huntee then
				       manhunt.huntee = ""
				       minetest.chat_send_player(name, param .. " has been removed from the huntee role.")
				    else
				       minetest.chat_send_player(name, param .. " is not a huntee.")
				    end
end})
minetest.register_chatcommand("removehunter", {
				 privs = {
				    manhuntadmin = true,
				 },
				 func = function(name, param)
				    if hasValue(manhunt.hunters, param) then
				       index = indexOf(manhunt.hunters, param)
				       if index == nil then
					  minetest.chat_send_player(name, "Error removing " .. param .. " from the hunter list: maybe try again?")
					  return
				       end
				       table.remove(manhunt.hunters, index)
				       minetest.chat_send_player(name, param .. " has been successfully removed from the hunter role.")
				       return
				    else
				       minetest.chat_send_player(name, param .. " is either not online or not in the hunter list.")
				       return
				    end
end})

minetest.register_chatcommand("addhunter", {
				 privs = {
				    manhuntadmin = true,
				 },
				 func = function(name, param)
				    if hasValue(manhunt.hunters, param) then
				       minetest.chat_send_player(name, "The specified player is already a hunter.")
				       return
				    elseif not hasValue(minetest.get_connected_players(), minetest.get_player_by_name(param)) then
				       minetest.chat_send_player(name, "No such player found online.")
				       return
				    elseif param == manhunt.huntee then
				       minetest.chat_send_player(name, "A huntee cannot be a hunter at the same time.")
				    end
				    
				    manhunt.hunters[#manhunt.hunters + 1] = param
				    minetest.chat_send_player(name, "Successfully added " .. param .. " as a hunter.")
end})				    
				    
minetest.register_chatcommand("startmanhunt", {
				 privs = {
				    manhuntadmin = true
				 },
				 func = function(name, param)
				    if #manhunt.hunters ~= 0 and manhunt.huntee ~= "" then
				       manhunt.manhunt = true
				       for _, i in pairs(manhunt.hunters) do
					  player = minetest.get_player_by_name(i)
					  if not player:get_inventory():contains_item("main", "manhunt:compass_off") and not player:get_inventory():contains_item("main", "manhunt:compass_on") then
					     minetest.get_player_by_name(i):get_inventory():add_item("main", ItemStack("manhunt:compass_off"))
					  end
				       end
				       minetest.chat_send_all("The manhunt has just started. Make sure to start running, speedrunner!")
				    else
				       minetest.chat_send_player(name, "Not enough participants to start the manhunt")
				    end
end})

minetest.register_chatcommand("stopmanhunt", {
				 privs = {
				    manhuntadmin = true
				 },
				 func = function(name, param)
				    if manhunt.manhunt == true then
				       minetest.chat_send_all("The manhunt has been halted.")
				    else
				       minetest.chat_send_player(name, "The manhunt has not been started yet.")
				    end
end})
minetest.register_chatcommand("hunters", {
				 privs = {
				    manhuntadmin = true,
				 },
				 func = function(name, param)
				    hunters = ""
				    if manhunt.hunters == nil then
				       return
				    end
				    
				    for k, hunter in pairs(manhunt.hunters) do
				       hunters = hunters .. hunter .. "\n"
				    end
				    minetest.chat_send_all(hunters)
end})

minetest.after(1, calc_compass_direction)

minetest.register_on_dieplayer(function(player, reason)
      if player:get_player_name() == manhunt.huntee and manhunt.manhunt == true then
	 manhunt.manhunt = false
	 minetest.chat_send_all("The manhunt has finished. The hunters won.")
      end      
end)
